MOTHERDUCK_READ_TOKEN = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJlbWFpbCI6InNvaHJhdWJwYXp1a2lAZ21haWwuY29tIiwic2Vzc2lvbiI6InNvaHJhdWJwYXp1a2kuZ21haWwuY29tIiwicGF0IjoiSklBM2UwLXctUkxUWXRoYXR0b0s5MnUxWXAxX0I3bGxzWjNIUkR0ZzRmVSIsInVzZXJJZCI6IjYzMzFhOGM5LWQ4YTctNDQzOS1iMjI4LTYzMTEzMTM2MWU5NyIsImlzcyI6Im1kX3BhdCIsInJlYWRPbmx5Ijp0cnVlLCJ0b2tlblR5cGUiOiJyZWFkX3NjYWxpbmciLCJpYXQiOjE3NTk3ODA5Nzh9.tYlhhvZTGJLODSVJAIzXPZx1bz5xoROqhvOTfH_qAqU'
